summaryrefslogtreecommitdiff
path: root/src/lib
diff options
context:
space:
mode:
Diffstat (limited to 'src/lib')
-rw-r--r--src/lib/timer.cc40
-rw-r--r--src/lib/timer.h37
2 files changed, 62 insertions, 15 deletions
diff --git a/src/lib/timer.cc b/src/lib/timer.cc
index e0cd93d7a..2e138aabf 100644
--- a/src/lib/timer.cc
+++ b/src/lib/timer.cc
@@ -1,5 +1,5 @@
/*
- Copyright (C) 2012 Carl Hetherington <cth@carlh.net>
+ Copyright (C) 2012-2019 Carl Hetherington <cth@carlh.net>
This file is part of DCP-o-matic.
@@ -30,6 +30,7 @@
#include "i18n.h"
using namespace std;
+using boost::optional;
/** @param n Name to use when giving output */
PeriodTimer::PeriodTimer (string n)
@@ -58,34 +59,55 @@ StateTimer::StateTimer (string n, string s)
_state = s;
}
+StateTimer::StateTimer (string n)
+ : _name (n)
+{
+
+}
+
+void
+StateTimer::set (string s)
+{
+ set_internal (s);
+}
+
/** @param s New state that the caller is in */
void
-StateTimer::set_state (string s)
+StateTimer::set_internal (optional<string> s)
{
double const last = _time;
struct timeval t;
gettimeofday (&t, 0);
_time = seconds (t);
- if (_totals.find (s) == _totals.end ()) {
- _totals[s] = 0;
+ if (s && _counts.find(*s) == _counts.end()) {
+ _counts[*s] = Counts();
}
- _totals[_state] += _time - last;
+ if (_state) {
+ _counts[*_state].total_time += _time - last;
+ _counts[*_state].number++;
+ }
_state = s;
}
+void
+StateTimer::unset ()
+{
+ set_internal (optional<string>());
+}
+
/** Destroy StateTimer and generate a summary of the state timings on cout */
StateTimer::~StateTimer ()
{
- if (_state.empty ()) {
+ if (!_state) {
return;
}
- set_state (N_(""));
+ unset ();
cout << _name << N_(":\n");
- for (map<string, double>::iterator i = _totals.begin(); i != _totals.end(); ++i) {
- cout << N_("\t") << i->first << " " << i->second << N_("\n");
+ for (map<string, Counts>::iterator i = _counts.begin(); i != _counts.end(); ++i) {
+ cout << N_("\t") << i->first << " " << i->second.total_time << " " << i->second.number << " " << (i->second.total_time / i->second.number) << N_("\n");
}
}
diff --git a/src/lib/timer.h b/src/lib/timer.h
index 9ea95c720..bd4e652f8 100644
--- a/src/lib/timer.h
+++ b/src/lib/timer.h
@@ -1,6 +1,6 @@
/*
- Copyright (C) 2012 Carl Hetherington <cth@carlh.net>
- Copyright (C) 2000-2007 Paul Davis
+ Copyright (C) 2012-2019 Carl Hetherington <cth@carlh.net>
+
This file is part of DCP-o-matic.
DCP-o-matic is free software; you can redistribute it and/or modify
@@ -26,6 +26,7 @@
#define DCPOMATIC_TIMER_H
#include <sys/time.h>
+#include <boost/optional.hpp>
#include <string>
#include <map>
@@ -59,20 +60,44 @@ private:
class StateTimer
{
public:
+ explicit StateTimer (std::string n);
StateTimer (std::string n, std::string s);
~StateTimer ();
- void set_state (std::string s);
+ void set (std::string s);
+ void unset ();
+
+ std::string name () const {
+ return _name;
+ }
+
+ class Counts
+ {
+ public:
+ Counts ()
+ : total_time (0)
+ , number (0)
+ {}
+
+ double total_time;
+ int number;
+ };
+
+ std::map<std::string, Counts> counts () const {
+ return _counts;
+ }
private:
+ void set_internal (boost::optional<std::string> s);
+
/** name to add to the output */
std::string _name;
/** current state */
- std::string _state;
+ boost::optional<std::string> _state;
/** time that _state was entered */
double _time;
- /** time that has been spent in each state so far */
- std::map<std::string, double> _totals;
+ /** total time and number of entries for each state */
+ std::map<std::string, Counts> _counts;
};
#endif